@@ -16,12 +16,12 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import print_function
-from __future__ import absolute_import
-
 import argparse
 import os
 import tempfile
+import unittest
+import io
+
 try:
     from contextlib import redirect_stderr  # noqa: H302
 except ImportError:
@@ -34,12 +34,6 @@ except ImportError:
         yield
         sys.stderr = old_target
 
-try:
-    import unittest
-except ImportError:
-    import unittest2 as unittest
-
-import six
 
 from gitlab import cli
 import gitlab.v4.cli
@@ -61,7 +55,7 @@ class TestCLI(unittest.TestCase):
         self.assertEqual("class", cli.cls_to_what(Class))
 
     def test_die(self):
-        fl = six.StringIO()
+        fl = io.StringIO()
         with redirect_stderr(fl):
             with self.assertRaises(SystemExit) as test:
                 cli.die("foobar")
@@ -69,8 +63,8 @@ class TestCLI(unittest.TestCase):
         self.assertEqual(test.exception.code, 1)
 
     def test_parse_value(self):
-        ret = cli._parse_value('foobar')
-        self.assertEqual(ret, 'foobar')
+        ret = cli._parse_value("foobar")
+        self.assertEqual(ret, "foobar")
 
         ret = cli._parse_value(True)
         self.assertEqual(ret, True)
@@ -82,68 +76,71 @@ class TestCLI(unittest.TestCase):
         self.assertEqual(ret, None)
 
         fd, temp_path = tempfile.mkstemp()
-        os.write(fd, b'content')
+        os.write(fd, b"content")
         os.close(fd)
-        ret = cli._parse_value('@%s' % temp_path)
-        self.assertEqual(ret, 'content')
+        ret = cli._parse_value("@%s" % temp_path)
+        self.assertEqual(ret, "content")
         os.unlink(temp_path)
 
-        fl = six.StringIO()
+        fl = io.StringIO()
         with redirect_stderr(fl):
             with self.assertRaises(SystemExit) as exc:
-                cli._parse_value('@/thisfileprobablydoesntexist')
-            self.assertEqual(fl.getvalue(),
-                             "[Errno 2] No such file or directory:"
-                             " '/thisfileprobablydoesntexist'\n")
+                cli._parse_value("@/thisfileprobablydoesntexist")
+            self.assertEqual(
+                fl.getvalue(),
+                "[Errno 2] No such file or directory:"
+                " '/thisfileprobablydoesntexist'\n",
+            )
             self.assertEqual(exc.exception.code, 1)
 
     def test_base_parser(self):
         parser = cli._get_base_parser()
-        args = parser.parse_args(['-v', '-g', 'gl_id',
-                                  '-c', 'foo.cfg', '-c', 'bar.cfg'])
+        args = parser.parse_args(
+            ["-v", "-g", "gl_id", "-c", "foo.cfg", "-c", "bar.cfg"]
+        )
         self.assertTrue(args.verbose)
-        self.assertEqual(args.gitlab, 'gl_id')
-        self.assertEqual(args.config_file, ['foo.cfg', 'bar.cfg'])
+        self.assertEqual(args.gitlab, "gl_id")
+        self.assertEqual(args.config_file, ["foo.cfg", "bar.cfg"])
 
 
 class TestV4CLI(unittest.TestCase):
     def test_parse_args(self):
         parser = cli._get_parser(gitlab.v4.cli)
-        args = parser.parse_args(['project', 'list'])
-        self.assertEqual(args.what, 'project')
-        self.assertEqual(args.action, 'list')
+        args = parser.parse_args(["project", "list"])
+        self.assertEqual(args.what, "project")
+        self.assertEqual(args.whaction, "list")
 
     def test_parser(self):
         parser = cli._get_parser(gitlab.v4.cli)
-        subparsers = None
-        for action in parser._actions:
-            if type(action) == argparse._SubParsersAction:
-                subparsers = action
-                break
+        subparsers = next(
+            action
+            for action in parser._actions
+            if isinstance(action, argparse._SubParsersAction)
+        )
         self.assertIsNotNone(subparsers)
-        self.assertIn('project', subparsers.choices)
+        self.assertIn("project", subparsers.choices)
 
-        user_subparsers = None
-        for action in subparsers.choices['project']._actions:
-            if type(action) == argparse._SubParsersAction:
-                user_subparsers = action
-                break
+        user_subparsers = next(
+            action
+            for action in subparsers.choices["project"]._actions
+            if isinstance(action, argparse._SubParsersAction)
+        )
         self.assertIsNotNone(user_subparsers)
-        self.assertIn('list', user_subparsers.choices)
-        self.assertIn('get', user_subparsers.choices)
-        self.assertIn('delete', user_subparsers.choices)
-        self.assertIn('update', user_subparsers.choices)
-        self.assertIn('create', user_subparsers.choices)
-        self.assertIn('archive', user_subparsers.choices)
-        self.assertIn('unarchive', user_subparsers.choices)
+        self.assertIn("list", user_subparsers.choices)
+        self.assertIn("get", user_subparsers.choices)
+        self.assertIn("delete", user_subparsers.choices)
+        self.assertIn("update", user_subparsers.choices)
+        self.assertIn("create", user_subparsers.choices)
+        self.assertIn("archive", user_subparsers.choices)
+        self.assertIn("unarchive", user_subparsers.choices)
 
-        actions = user_subparsers.choices['create']._option_string_actions
-        self.assertFalse(actions['--description'].required)
+        actions = user_subparsers.choices["create"]._option_string_actions
+        self.assertFalse(actions["--description"].required)
 
-        user_subparsers = None
-        for action in subparsers.choices['group']._actions:
-            if type(action) == argparse._SubParsersAction:
-                user_subparsers = action
-                break
-        actions = user_subparsers.choices['create']._option_string_actions
-        self.assertTrue(actions['--name'].required)
+        user_subparsers = next(
+            action
+            for action in subparsers.choices["group"]._actions
+            if isinstance(action, argparse._SubParsersAction)
+        )
+        actions = user_subparsers.choices["create"]._option_string_actions
+        self.assertTrue(actions["--name"].required)
